/*

==============================证件照制作高精版 预览接口_免费==============================
*/
package com.WXPIdphotoHeigh;

import com.alibaba.fastjson.JSON; // 依赖包下载地址 https://mvnrepository.com/

import java.security.MessageDigest;
import javax.imageio.ImageIO;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/*
 说明：
     此代码是一份调用接口调用示例，
     WXAIPhotoAPI也只是一份调用示例的class，并不是接口本身，
     请不要把WXAIPhotoAPI的参数，当成接口本身的参数
*/

public class Preview {


    // 证件照制作高精版预览接口_免费
    private final String IDPhotoAPIServer = "https://idphotohq-open.segapi.com";
    private final String IDPhotoAPIUri = "/preview";
    private final int MaxPhotoEdge = 2000;


    private String base64(byte[] bytes) {
        return Base64.getEncoder().encodeToString(bytes);
    }

    public byte[] resizeAndReadImage(File srcFile) throws IOException {
        ByteArrayOutputStream out = null;
        try {
            // 构造Image对象  
            BufferedImage src = ImageIO.read(srcFile);
            int width = src.getWidth();
            int height = src.getHeight();

            int maxEdge = Math.max(width, height);

            // 接口要求图片最长边不超过1500个像素，超过的话，需要先做缩放
            if (maxEdge <= this.MaxPhotoEdge) {
                FileInputStream inputFile = new FileInputStream(srcFile);
                byte[] buffer = new byte[(int) srcFile.length()];
                inputFile.read(buffer);
                inputFile.close();
                return buffer;
            }
            float ratio = (float) this.MaxPhotoEdge / (float) maxEdge;
            int newWidth = (int) ((float) width * ratio);
            int newHeight = (int) ((float) height * ratio);

            BufferedImage tag = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_INT_RGB);
            tag.getGraphics().drawImage(src, 0, 0, newWidth, newHeight, null);

            out = new ByteArrayOutputStream();
            ImageIO.write(tag, "JPEG", out);
            return out.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (out != null) {
                out.close();
            }
        }
        return null;
    }


    private String httpPost(String httpUrl, Map<String, String> headers, String postData) {

        HttpURLConnection connection = null;
        InputStream is = null;
        OutputStream os = null;
        BufferedReader br = null;
        String result = null;
        try {
            URL url = new URL(httpUrl);

            connection = (HttpURLConnection) url.openConnection();
            if (url.getProtocol().toLowerCase().equals("https")) {
                HttpsURLConnection https = (HttpsURLConnection) connection;
                https.setHostnameVerifier(DO_NOT_VERIFY);
                //con = https;
            }

            connection.setRequestMethod("POST");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);

            connection.setDoOutput(true);

            if (headers != null) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }

            os = connection.getOutputStream();
            os.write(postData.getBytes());

            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                StringBuffer sbf = new StringBuffer();
                String temp = null;
                while ((temp = br.readLine()) != null) {
                    sbf.append(temp);
                    sbf.append("\r\n");
                }
                result = sbf.toString();

            } else {
                System.out.println("bad status code:" + connection.getResponseCode());
                Map<String, List<String>> map = connection.getHeaderFields();
                for (Map.Entry<String, List<String>> entry : map.entrySet()) {
                    System.out.println("Key : " + entry.getKey() +
                            " ,Value : " + entry.getValue());
                }
                is = connection.getErrorStream();
                if (is != null) {
                    br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                    StringBuffer sbf = new StringBuffer();
                    String temp = null;
                    while ((temp = br.readLine()) != null) {
                        sbf.append(temp);
                        sbf.append("\r\n");
                    }
                    result = sbf.toString();
                }

            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != os) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 断开与远程地址url的连接
            connection.disconnect();
        }
        return result;
    }

    public static byte[] readInputStream(InputStream inputStream) throws IOException {
        byte[] buffer = new byte[1024];
        int len = 0;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        while ((len = inputStream.read(buffer)) != -1) {
            bos.write(buffer, 0, len);
        }
        bos.close();
        return bos.toByteArray();
    }

    public static void downloadFromUrl(String urlStr, File targetFile) throws IOException {
        URL url = new URL(urlStr);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setConnectTimeout(10 * 1000);
        conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");

        InputStream inputStream = conn.getInputStream();
        //获取自己数组
        byte[] getData = readInputStream(inputStream);

        FileOutputStream fos = new FileOutputStream(targetFile);
        fos.write(getData);
        if (fos != null) {
            fos.close();
        }
        if (inputStream != null) {
            inputStream.close();
        }

        System.out.println("info:" + url + " download success");

    }

    final static HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {

        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };

    private byte[] md5Hash(String dataStr) throws Exception {

        dataStr = dataStr;
        MessageDigest md5 = MessageDigest.getInstance("MD5");
        md5.update(dataStr.getBytes("UTF8"));
        byte bytes[] = md5.digest();
        return bytes;
    }


    private String extractValueFromJsonString(String jsonString, String key) {
        String regex = "\"" + key + "\":([^(,|\\}|\\])]*)";
        Matcher matcher = Pattern.compile(regex).matcher(jsonString);
        while (matcher.find()) {
            String val = matcher.group(1);
            val = val.trim();
            if (val.startsWith("\"")) {
                val = val.substring(1);
            }
            if (val.endsWith("\"")) {
                val = val.substring(0, val.length() - 1);
            }
            return val;
        }
        return null;

    }

    private String apiCall(String apiServer, String apiUri, File imageFile, String Appcode, Map<String, Object> args) {
        try {

            byte[] buffer = resizeAndReadImage(imageFile);
            if (buffer == null) {
                return null;
            }
            // BASE64Encoder b64Encoder = new BASE64Encoder();
            String imageBase64 = base64(buffer);
            String fileName = imageFile.getName();
            String fileExt = fileName.substring(fileName.lastIndexOf(".") + 1);
            String photoType = fileExt.replace(".", "");


            Map bodyJson = new HashMap();
            bodyJson.put("photo", imageBase64);
            bodyJson.put("type", photoType);


            Map<String, Object> combineResultMap = new HashMap<String, Object>();
            combineResultMap.putAll(bodyJson);
            combineResultMap.putAll(args);


            String body = JSON.toJSONString(combineResultMap);

            String bodyMd5 = base64(md5Hash(body));


            String accept = "application/json";
            String contentType = "application/octet-stream; charset=utf-8";

            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Accept", accept);
            headers.put("Authorization", "APPCODE " + Appcode);
            headers.put("Content-Type", contentType);
            headers.put("Content-MD5", bodyMd5);


            String result = httpPost(apiServer + apiUri, headers, body);
            if (result == null) {
                System.out.println("httpPost call fail");
                return null;
            }

            System.out.println("result:" + result);
            return result;


        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }

    // 证件照制作接口
    public Boolean idPhotoMakePre(File imageFile, String specID, String[] bk_list, File saveToFile, String Appcode) {
        System.out.println("证件照制作高精版预览接口_免费 ...");
        try {
            Map<String, Object> args = new HashMap<String, Object>();
            args.put("spec", specID);
            args.put("bk_list", bk_list);

            /*
            其余参数（选填）：
                beauty_degree: 美颜级别（主要控制亮度平衡），取值[1.0-5.0]之间的浮点，值越大，亮度越大，超过5.0的值无效；
                face_ratio: 脸部占比，取值[0.1-1.0)之间的浮点数，用于控制人脸在证件照图中的占比；
                output_quality: 输出证件照图片质量， 取值[0-100]之间的整形；
                dpi：输出证件照图片dpi，整形，比如：300， 350
            注意：face_ratio，output_quality，dpi三个参数会改变证件照输出的标准。
            */
            String result = apiCall(this.IDPhotoAPIServer,
                    this.IDPhotoAPIUri,
                    imageFile,
                    Appcode, args);
            if (result == null) {
                return false;
            }


            // 正确做法是使用json解析库，
            // 这里为了减少依赖，直接使用正则表达式解析
            String status = extractValueFromJsonString(result, "status");
            if (!status.equals("0")) {
                System.out.println("bad status:" + status);
                return false;
            }
            String url = extractValueFromJsonString(result, "result");
            if (url != null) {
                downloadFromUrl(url, saveToFile);
            }

            return true;
        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }
        return false;
    }


    public static void main(String[] args) {
        
        /*
         说明：
             此代码是一份调用接口调用示例，
             WXAIPhotoAPI也只是一份调用示例的class，
             并不是接口本身，
             请不要把WXAIPhotoAPI的参数，当成接口本身的参数
        */
        Preview api_test = new Preview();

        String imageFileName = "1.jpg";
        String Appcode = "你的appcode";
        File imageFile = new File(imageFileName);

        String[] bk_list = {"white", "red", "#887799"};
        // 证件照制作接口函数调用
        File dstFile = new File("2.jpg");
        api_test.idPhotoMakePre(imageFile, // 原图
                "1", // 规格ID，可以通过使用文档找到期望的ID（文档在“证件照制作接口”页面的“资料下载”中可以找到）
                bk_list, // 背景颜色
                dstFile, // 证件照结果图，这个参数并不是接口要求的输入参数，只是将接口返回的图片，保存到这个文件中，方便查看
                Appcode
        );

        return;
    }

}